#!/usr/bin/perl -w --

##  tunezd -- Tunez daemon
##
##  Hacked for use with the Tunez project
##
##  ------------------------------------
##
##  jbplayd.pl -- Jukebox Player daemon
##	by Roland Steinbach <roland@ratisbona.com>
##  small tweaks by Dave O'Neill <dmo@acm.org>
##  
##  This program is released under the GPL (see LICENSE in this archive)
##

use DBI;

## Some "about" info
$programName 	= "Tunez daemon";
$version		= "0.7";

## MP3 Player binaries and switches.
my $binary		= "/usr/local/bin/mpg123";
my @args  		= ("-r","44100","-q");

## IceCast hooks
my $iceCastDir = "/usr/bin";

## Other Vars.
my $DEBUG = 0; 		# 1 for debug info to stdout, 0 for none.
my $delay = 0; 	# Default delay for random playing

my $query="";	
my $sth=0;		
my $entries=0;		
my $position=0;		
my $id=0;			
my $play=0;			

my $artist="";	        # Artist
my $songtitle="";	# Song title
my $origin="";		# the originating host
my $filename="";	# Filename
my $rndtime = 0;	#
my $rndflag = 0;	#
my $playtime=0;
my $votes=0;
my $timesPlayed=0;
##
## Get our command line options
## This uses a lovely perl hack from the Camel book to 
## simulate a switch statement. 
while (defined($_ = shift)) {

	print "Processing arg: $_\n" if $DEBUG; 

	m/(-d|--delay)/ && do {	
		## Set the delay
		defined($delay = shift) || &DieNicely("$_ option requires an argument");
		print "Setting delay to $delay\n" if $DEBUG;
		next;
	};
	m/(-h|--help)/ && do {
		## Display help message
		&Usage();
		exit();
	};	
	m/(-v|--verbose|--debug)/ && do {
		## Set verbose output
		$DEBUG = 1;
		print "Setting DEBUG on!\n";
	};
	m/(-s|--stream)/ && do {
		## Set streaming on.
		$stream = 1;
		print "Setting streaming on\n" if $DEBUG;
	};
}	
			

## Print some notification info
print "$programName (v. $version) starting in background\n";

## Now fork and become a daemon unless we're debugging
unless ($DEBUG) {
	fork && exit;
	setpgrp();
}

## Get ourselves a database handle.
my ($dbh) = DBI->connect("DBI:mysql:tunez:localhost","tunez","");
print "Connected to database\n" if $DEBUG;

## Loop forever 
while () {
	## Read entries from the table


    $query = "select *, count(*) as votes from queue LEFT JOIN songs using(song_id) group by filename order by votes DESC, timestamp";
    $sth=$dbh->prepare($query);
    $entries = $sth->execute();
    if ($entries == 0) {	
    	## We have no entries in the queue, so we're going
	## to pick a random song
        ## print "No entry in song queue\n" if $DEBUG;
        $sth->finish();
		
	if ($rndflag == 0) {
		## if we are restarting the random countdown, do this
	    	$rndflag = 1;
	    	$rndtime = time();
		} 
	else {
		## else, we're continuing a countdown and no queued songs 
		## have arrived
   	if ((time()-$rndtime) < $delay) {
		## If we haven't slept the full time yet, sleep some more
	       	sleep int($delay / 10);
	    	} 
	else {
		## Else, we've slept long enough -- pick a random song.
		$query="SELECT song_id FROM songs";
		$sth=$dbh->prepare($query);
		$entries = 0;
		$sth->execute();
		while (($origin)=$sth->fetchrow_array) {
		    $entries++;
		    }
		$sth->finish();
		$song_id = int (1 + rand($entries));
                print "Randomly selected song $id/$entries.\n" if $DEBUG;
		$entries=1;
		$origin = "RandomPlay";
	    	}
	}	
    } else {
		## Songs were found in the queue, so we shall use them.
		
		($queue_id, $song_id) = $sth->fetchrow_array();
		print "About to play song $queue_id from queue\n" if $DEBUG;
        	$sth->finish();

		#Remove the song we're going to play from the top of queue
		$query="DELETE FROM queue WHERE song_id = $song_id";
		$sth=$dbh->prepare($query);
		$sth->execute();
		$sth->finish();

		## Now, renumber the remaining queue items
		## This replaces some ugly stuff with temp tables.
		#$query="UPDATE queue SET position = position-1";
		#$sth=$dbh->prepare($query);
		#$sth->execute();
		#$sth->finish();
	}	
		## We've done it... now let's play some music!
	if ($entries != 0) {
		## Get the song info from the songs database
        	$query = "SELECT artist,songtitle,filename FROM songs WHERE song_id = $song_id";
	        $sth=$dbh->prepare($query);
        	if (($sth->execute()) == 0) {
            		print "BOAAHHH, NOT FOUND?!";
        		} else {
	        ($artist,$songtitle,$filename) = $sth->fetchrow_array();
		$sth->finish();
		
		## Update now-playing record
		$query="DELETE FROM np";
		$sth=$dbh->prepare($query);
		$sth->execute();
		$sth->finish();
		$query="INSERT INTO np VALUES ('$song_id')";
		$sth=$dbh->prepare($query);
		$sth->execute();
		$sth->finish();
		
		# update timesPlayed
		
		if ($rndflag != 1) {
		  $query="UPDATE songs SET timesPlayed=timesPlayed + 1 WHERE song_id = $song_id";
		  $sth=$dbh->prepare($query);
		  $sth->execute();
		  $sth->finish();
 	 	}
	
		## Build up our args list, then stick the
		## binary on the front of it and the filename on the end.
		@execargs = @args;
		unshift (@execargs,$binary);
		push (@execargs,$filename);
		
		$rndflag=0;
		print "Now Playing: $artist/$songtitle ($filename)\n" if $DEBUG;
		if ($stream) {
			## play via IceCast
			$cmd = "$iceCastDir/shout localhost -P hackme -x -n 'This is Tunez' '$filename'";
			print "Commandline: $cmd";
			system($cmd);
		} else {
			print "Commandline:" ,join(" ",@execargs),"\n" if $DEBUG;
    		system @execargs;
		}
		}
	}
}


##
##  Put out a nice error message and exit 
##

sub DieNicely {
	$arg = shift;
	print STDERR "$0: $arg\n";
	exit();
}

##
##  Print out a usage message
##
sub Usage {
	print STDERR<<END;
$0: $programName (v.$version); 

 -d,--delay <n>            Delay n seconds before playing random song.
 -h,--help                 Display this usage message.
 -v,--verbose,--debug      Display verbose output
 -s,--stream               Stream using icecast

END

}
